-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = "iWait";
iWait = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0");

local band, bor = bit.band, bit.bor;

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

-----------------------------------------
-- Variables, functions and colors
-----------------------------------------

local COLOR_RED  = "|cffff0000%s|r";
local COLOR_GREEN= "|cff00ff00%s|r";
local COLOR_YELLOW= "|cffffff00%s|r";

local ICON_PVP = "Interface\\BattlefieldFrame\\UI-Battlefield-Icon";
local ICON_LFG_ON = "Interface\\Addons\\iWait\\Images\\LFG-On";
local ICON_LFG_OFF = "Interface\\Addons\\iWait\\Images\\LFG-Off";

local QUEUESTATE_ACTIVE = 3;
local QUEUESTATE_CONFIRM = 2;
local QUEUESTATE_QUEUED = 1;
local QUEUESTATE_NONE = 0;

local QueueState = 0; -- sets the queue state, where 0 is either none or error (and also none)
local QueueType = 0; -- ORed: 0 is none, 1 is pve, 2 is raid, 4 is pvp, 8 is arena. this allows me to get rid of thousands booleans

-----------------------------
-- Setting up the feed
-----------------------------

local function Feed_OnClick(anchor, button)
	-- if not queued
	if( QueueState < QUEUESTATE_QUEUED ) then
		if( button == "LeftButton" and IsAltKeyDown() ) then
			ToggleFrame(RaidParentFrame); -- ToggleLFRParentFrame() doesn't seem to work for some reason
		elseif( button == "LeftButton" ) then
			ToggleLFDParentFrame();
		elseif( button == "RightButton" ) then
			TogglePVPFrame();
		end
	-- if queued/confirm/active
	else
		-- if pvp
		if( band(QueueType, 0x04) == 4 ) then
			if( button == "LeftButton" and QueueState == QUEUESTATE_ACTIVE ) then
				ToggleWorldStateScoreFrame();
			elseif( button == "RightButton" ) then
				ToggleDropDownMenu(1, nil, MiniMapBattlefieldDropDown, anchor, 0, 0);
				GameTooltip:Hide();
			end
		-- else pve
		else
			if( button == "RightButton" ) then
				ToggleDropDownMenu(1, nil, MiniMapLFGFrameDropDown, anchor, 0, 0);
				LFGSearchStatus:Hide();
			end
		end
	end
end

local function Feed_OnEnter(anchor)
	-- if not queued
	if( QueueState < QUEUESTATE_QUEUED ) then
		-- nothing to do
	-- if queued/confirm/active
	else
		if( DropDownList1:IsVisible() ) then
			return;
		end
			
		-- if pve
		if( band(QueueType, 0x01) == 1 ) then
			LFGSearchStatus:SetParent(anchor);
			LFGSearchStatus:SetClampedToScreen(true);
			LFGSearchStatus:SetPoint("TOPRIGHT", anchor, "BOTTOM", 175, 0);
			LFGSearchStatus:Show();
		-- if pvp
		else
			GameTooltip:SetOwner(anchor, "ANCHOR_BOTTOM", 0, 0);
			GameTooltip:SetClampedToScreen(true);
			
			if( MiniMapBattlefieldFrame.tooltip ) then
				GameTooltip:SetText(MiniMapBattlefieldFrame.tooltip);
			end
			
			anchor:SetScript("OnUpdate", function()
				PVP_UpdateStatus(1);
				if( MiniMapBattlefieldFrame.tooltip ) then
					GameTooltip:SetText(MiniMapBattlefieldFrame.tooltip, nil, nil, nil, nil, 1);
				end
			end);
		end
		
	end
end

local function Feed_OnLeave(anchor)
	-- if queued/confirm/active
	if( QueueState > QUEUESTATE_NONE ) then
		-- if pve
		if( band(QueueType, 0x01) == 1 ) then
			LFGSearchStatus:Hide();
		-- if pvp
		else
			GameTooltip:Hide();
			anchor:SetScript("OnUpdate", nil);
		end
	end
end

iWait.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	OnClick = Feed_OnClick,
	OnEnter = Feed_OnEnter,
	OnLeave = Feed_OnLeave,
});

------------------------------------------
-- OnInitialize and Reset
------------------------------------------

function iWait:OnInitialize()
	self:Reset();
	-- pve
	self:RegisterEvent("LFG_UPDATE", "LFGHandler");
	self:RegisterEvent("LFG_QUEUE_STATUS_UPDATE", "LFGHandler");
	self:RegisterEvent("LFG_PROPOSAL_UPDATE", "LFGHandler");
	self:RegisterEvent("LFG_PROPOSAL_SUCCEEDED", "LFGHandler");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "LFGHandler");
	-- pvp
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "PvPHandler");
end

function iWait:Reset()
	self.Feed.text = "";
	self.Feed.icon = ICON_LFG_OFF;
	QueueType = 0;
	QueueState = QUEUESTATE_NONE;
end

------------------------------------------
-- The PvP Handler
------------------------------------------

function iWait:PvPHandler()
	-- there are sometimes more than one pvp queues simultanly, f. e. Warsong and Arathi
	-- our task is to check for the "highest". that means active is higher than confirm, confirm is higher than queued, ...
	local state = QUEUESTATE_NONE;
	
	for i = 1, MAX_WORLD_PVP_QUEUES do
		local status, _, _, _, _, teamSize, _ = GetBattlefieldStatus(i);
		
		local thisState = QUEUESTATE_NONE;
		
		if( status == "active" ) then
			thisState = QUEUESTATE_ACTIVE;
		elseif( status == "confirm" ) then
			thisState = QUEUESTATE_CONFIRM;
		elseif( status == "queued" ) then
			thisState = QUEUESTATE_QUEUED;
		end
		
		if( thisState > state ) then
			state = thisState;
		end
		
		-- teamSize determines if we are queued for arena or playing arena
		if( teamSize ~= 0 ) then
			QueueType = bor(QueueType, 8);
		end
	end
	
	if( state == QUEUESTATE_ACTIVE ) then
		QueueType = bor(QueueType, 4);
		self.Feed.icon = ICON_PVP;
		self.Feed.text = (COLOR_GREEN):format(L["In Progress"]);
		
	elseif( state == QUEUESTATE_CONFIRM ) then
		QueueType = bor(QueueType, 4);
		self.Feed.icon = ICON_PVP;
		
		if( band(QueueType, 0x08) == 8 ) then
			self.Feed.text = (COLOR_YELLOW):format(L["Enter Arena"]);
		else
			self.Feed.text = (COLOR_YELLOW):format(L["Enter Battlefield"]);
		end
		
	elseif( state == QUEUESTATE_QUEUED ) then
		QueueType = bor(QueueType, 4);
		self.Feed.icon = ICON_PVP;
		
		if( band(QueueType, 0x08) == 8 ) then
			self.Feed.text = (COLOR_RED):format(L["Arena"]);
		else
			self.Feed.text = (COLOR_RED):format(L["Battlefield"]);
		end
		
	else
		self:Reset();
	end
	
	QueueState = state;
end

------------------------------------
-- The LFG Handler
------------------------------------

-- the LFG API doesn't return how much people are in the group, it returns how much people are currently missing
-- in a raid f. e. tanks = 2, heal = 5 (6), dps = 17 (if we are a healer and queued 1 second ago)
-- so need would be 0 if all people of a role are present, that's where the following two functions are in place
-- LFGColor checks if the need is less than a specific check value and if yes, encolors the text green, else red
local function LFGColor(check, currneed, text)
	if( currneed < check ) then
		return (COLOR_GREEN):format(text);
	else
		return (COLOR_RED):format(text);
	end
end
-- LFGText prints out the "THD" or "THDDD" text in red/green letters to display, what's missing
-- we have the opportunity to add icons instead of letters here, at a later point
local function LFGText(tankneed, healneed, dpsneed)
	-- if raid
	if( band(QueueType, 0x02) == 2 ) then
		return ("%s: %s%s%s"):format( L["LFR"],
			LFGColor(1, tankneed, L["T"]),
			LFGColor(1, healneed, L["H"]),
			LFGColor(1, dpsneed, L["D"])
		);
	-- if just pve
	else
		return ("%s: %s%s%s%s%s"):format( L["LFD"],
			LFGColor(1, tankneed, L["T"]),
			LFGColor(1, healneed, L["H"]),
			LFGColor(3, dpsneed, L["D"]), -- if missing dps are 2, 1 dps must be there! green here
			LFGColor(2, dpsneed, L["D"]), -- same but missing dps is 1
			LFGColor(1, dpsneed, L["D"])
		);
	end
end

function iWait:LFGHandler()
	local mode, submode = GetLFGMode();
	
	if( not mode ) then mode = "suspended" end
	if( not submode ) then submode = "nil" end
	--print(mode.." - "..submode);
	
	-- we obviously left the dungeon or the queue.
	if( not mode or mode == "suspended" or mode == "abandonedInDungeon") then
		self:Reset();
		return;
		
	elseif( mode == "lfgparty" ) then
		QueueState = QUEUESTATE_ACTIVE;
		self.Feed.icon = ICON_LFG_ON;
		self.Feed.text = (COLOR_GREEN):format(L["In Progress"]);
		return;
		
	elseif( mode == "proposal" ) then
		QueueState = QUEUESTATE_CONFIRM;
		self.Feed.icon = ICON_LFG_ON;
		
		if( submode and submode == "unaccepted" ) then
			self.Feed.text = (COLOR_GREEN):format(L["Enter Dungeon"]);
		else
			self.Feed.text = (COLOR_YELLOW):format(L["Waiting..."]);
		end
		
	elseif( mode == "queued" ) then
		QueueState = QUEUESTATE_QUEUED;
		self.Feed.icon = ICON_LFG_ON;
		
		local hasData, _, tankNeed, healNeed, dpsNeed, iniType, _, _, _, _, _, _, _ = GetLFGQueueStats();
		
		if( hasData ) then
			QueueType = bor(QueueType, 1);
			
			if( iniType and iniType == 2 ) then
				QueueType = bor(QueueType, 2);
			end
			
			self.Feed.text = LFGText(tankNeed, healNeed, dpsNeed);
		else
			self.Feed.text = (COLOR_YELLOW):format(L["Preparing..."]);
		end
	end
end